unit ComPort;
interface
uses windows, sysutils, classes, forms;

const
  NO_PARITY     = 0;
  ODD_PARITY    = 1;
  EVEN_PARITY   = 2;
  MARK_PARITY   = 3;
  SPACE_PARITY  = 4;

  // merke: 1.5 Stopbits sind bei 6..8 Datenbits verboten
  STOPBITS_1    = 0;
  STOPBITS_2    = 2;


const
  buffermax = 16383; { fr den Empfangspuffer }

{ ComPort class }
type
  TComPort = class(TComponent)
  private                          { Private declarations }
    hPort          : DWORD;        {handle from OpenComm }
    nPort          : Integer;      { port #, 1-based }
    lBaud          : LongInt;      { baud rate }
    hLocal         : boolean;
    RxBuff         : array[0..buffermax] of AnsiChar;
    inPtr,
    outPtr         : integer;
    nParity        : integer;
    nStopbits      : integer;
                                   { functions }
    procedure SetBaud    (lBaudToSet: LongInt);
    procedure SetParity  (aParity: integer);
    procedure SetPort    (nPortToSet: Integer);
    procedure SetStop    (nStopbitsToSet: Integer);
    function  GetInCount : LongInt;
    function  IsOpen     : Boolean;

  public                   { Public declarations }
                           { Methods }
    dcbPort   : TDCB;      {device control block }
    dcbValid  : boolean;
    constructor Create  (AOwner: TComponent); override;
    destructor  Destroy; override;
    function    Open:    Boolean;
    function    Write   (const sData: AnsiString): DWORD;
    function    WriteB  (sData: byte): boolean;
    function    WritePB (pData: LPBYTE; len: DWORD): DWORD;
    function    Read:    AnsiChar;
    function    ReadB:   byte;
    procedure   Flush;
    procedure   Close;
    procedure   SetLocal(ifLocal: boolean);
    procedure   WaitForEvent (var Event: cardinal);
    procedure   AssignDTR(state: boolean);
    procedure   AssignRTS(state: boolean);

  published                { Published declarations }
    property Baud:     LongInt read lBaud     write SetBaud;
    property Port:     Integer read nPort     write SetPort;
    property Stopbits: Integer read nStopbits write SetStop;
    property InCount:  LongInt read GetInCount;
    property Opened:   Boolean read IsOpen;
    property Local:    boolean read hLocal    write SetLocal;
    property Parity:   Integer read nParity   write SetParity;
end;


procedure Register;


{----------------------------------------------------------------------------}
implementation
{$R *.DCR}

           { Register the component with the Delphi IDE }
procedure Register;
begin
  RegisterComponents('Communications', [TComPort]);
end;

                                 { Component constructor }
constructor TComPort.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);      { set default property values }
  hPort := INVALID_HANDLE_VALUE; { invalidate to start }
  lBaud := 9600;
  nParity:= NOPARITY;
  nStopbits:= STOPBITS_2;
  nPort := 1;
  inPtr := 0;
  outPtr:= 0;
end;


                       { Component destructor }
destructor TComPort.Destroy;
begin
  Close;               { close the com port (if open) }
  inherited Destroy;   { destroy ancestor class }
end;


                       { Return True if port is open }
function TComPort.IsOpen: Boolean;
begin
  Result := (hPort <> INVALID_HANDLE_VALUE);
end;


                       { Set the baud rate property }
procedure TComPort.SetBaud (lBaudToSet: LongInt);
begin
  if lBaudToSet <> lBaud
  then
  begin
	  lBaud := lBaudToSet;
    if IsOpen then begin Close; Open; end;
  end;
end;


                       { Set the Port parity }
procedure TComPort.SetParity  (aParity: integer);
begin
  if aParity <> nParity
  then
  begin
	 nParity := aParity;
   if IsOpen then begin Close; Open; end;
  end;
end;


                       { Set the Port property }
procedure TComPort.SetPort (nPortToSet: Integer);
begin
  if nPortToSet <> nPort
  then
  begin
    nPort := nPortToSet;
    if hPort <> INVALID_HANDLE_VALUE
    then  begin Close; Open; end;
  end;
end;


                       { Set the Stopbits property }
procedure TComPort.SetStop (nStopbitsToSet: Integer);
begin
  if (nStopbitsToSet = STOPBITS_1) or
     (nStopbitsToSet = STOPBITS_2)
  then nStopbits:= nStopbitsToSet
  else nStopbits:= STOPBITS_2;

  if IsOpen then begin Close; Open; end;
end;


                  { Opens the COM port, returns True if ok }
function TComPort.Open: Boolean;
var
  sCom : array[0..16] of byte;
  i    : integer;
  p    : integer;
  pb   : array[0..2] of integer;
begin
  result:= false;
                  { close port if open already }
  if hPort <> INVALID_HANDLE_VALUE then Close;

 	          { try to open the port }
  i:= 0;
  if nPort>=10
  then begin
         sCom[0]:= ord('\');
         sCom[1]:= ord('\');
         sCom[2]:= ord('.');
         sCom[3]:= ord('\');
         i:= 4;
       end;
  sCom[i]:= ord('C'); inc(i);
  sCom[i]:= ord('O'); inc(i);
  sCom[i]:= ord('M'); inc(i);

  p:= nPort;
  pb[2]:= p mod 10;  p:= p div 10;
  pb[1]:= p mod 10;  p:= p div 10;
  pb[0]:= p mod 10;

  if pb[0]<>0 then begin sCom[i]:= $30+pb[0]; inc(i); end;
  if pb[1]<>0 then begin sCom[i]:= $30+pb[1]; inc(i); end;
  sCom[i]:= $30+pb[2]; inc(i);
  sCom[i]:= 0;

  hPort := CreateFileA (@sCom,
                        GENERIC_READ or GENERIC_WRITE,
                        0,
                        nil,
                        OPEN_EXISTING,
                        FILE_ATTRIBUTE_NORMAL,
                        LongInt(0));

  if hPort = INVALID_HANDLE_VALUE then exit;

                  { set the baud rate and other parameters }
  if GetCommState(hPort, dcbPort)
  then begin
                  { fill in the fields of the structure }
         dcbPort.BaudRate := lBaud;
         dcbPort.ByteSize := 8;
         dcbPort.Parity   := nParity;
         dcbPort.StopBits := nStopbits;
         dcbValid:= SetCommState(hPort, dcbPort);
       end;
                      { return True if handle is valid }
  lBaud:= dcbPort.BaudRate;
  result := (hPort <> INVALID_HANDLE_VALUE) and dcbValid;
end;

procedure TComPort.AssignDTR(state: boolean);
begin
  if state
  then EscapeCommFunction(hPort, SETDTR)
  else EscapeCommFunction(hPort, CLRDTR);
end;

procedure TComPort.AssignRTS(state: boolean);
begin
  if state
  then EscapeCommFunction(hPort, SETRTS)
  else EscapeCommFunction(hPort, CLRRTS);
end;


                      { Close the COM port }
procedure TComPort.Close;
begin
  if hPort <> INVALID_HANDLE_VALUE then CloseHandle(hPort);
  hPort := INVALID_HANDLE_VALUE;
end;


       { Write a string out the COM port, return true if all chars written }
function TComPort.Write (const sData: AnsiString): DWORD;
var
  dwCharsWritten: DWORD;
  i: longint;
begin
  dwCharsWritten := 0;
  Result := 0;
  if hPort <> INVALID_HANDLE_VALUE
  then
  begin
    WriteFile(hPort, PAnsiChar(sData)^, Length(sData), dwCharsWritten, nil);
    result:= dwCharsWritten;
    if hLocal
    then begin
           for i:= 1 to dwCharsWritten do
           begin
             RxBuff[inPtr]:= sData[i];
             if inPtr<buffermax-1 then inc(inPtr);
           end;
         end;
  end;
end;


       { Write a single Byte out the COM port, return true if written }
function TComPort.WriteB (sData: byte): boolean;
var
  dwCharsWritten: DWORD;
  Buff: array[0..1] of byte;
begin
  Buff[0]:= sData;
  Buff[1]:= 0;
  dwCharsWritten := 0;
  Result := true;
  if hPort <> INVALID_HANDLE_VALUE
  then
  begin
    WriteFile(hPort, Buff, 1, dwCharsWritten, nil);
    result:= dwCharsWritten = 1;
    if hLocal
    then begin
           RxBuff[inPtr]:= AnsiChar(sData);
           if inPtr<buffermax-1 then inc(inPtr);
         end;
  end;
end;

   { Write a Arrayy of Bytee out the COM port, return number of written bytes }
function TComPort.WritePB (pData: LPBYTE; len: DWORD): DWORD;
var
  dwCharsWritten: DWORD;
  i : DWORD;
begin
  dwCharsWritten := 0;
  Result := 0;
  if hPort <> INVALID_HANDLE_VALUE
  then
  begin
    WriteFile(hPort, pData^, len, dwCharsWritten, nil);
    result:= dwCharsWritten;
    if hLocal
    then begin
           i:= 0;
           while i < dwCharsWritten do
           begin
             RxBuff[inPtr]:= AnsiChar(pData^);
             inc(pData);
             inc(i);
             if inPtr<buffermax-1 then inc(inPtr);
           end;
         end;
  end;
end;


                         { Return the number of bytes waiting in the queue }
function TComPort.GetInCount: LongInt;
var
  statPort: TCOMSTAT;
  dwErrorCode: DWord;
  i: LongInt;
begin
  i := 0;
  if hPort <> INVALID_HANDLE_VALUE
  then begin
         ClearCommError(hPort, dwErrorCode, @statPort);
         i:= statPort.cbInQue;
       end;
  if inPtr > outPtr
  then begin
         i:= i + inPtr - outPtr;
       end;
  Result:= i;
end;


function TComPort.Read: AnsiChar;
var
  Ch_Cnt,
  Ch_Read: DWord;
begin
  Result := #0;
  if inPtr > outPtr          { Puffer ist nicht leer, also daraus lesen }
  then begin
         Result:= RxBuff[outPtr];
	       inc(outPtr);
         exit;
       end;
                            { Puffer ist leer, also Windows bemhen... }
  if hPort = INVALID_HANDLE_VALUE then exit;
  Ch_Cnt:= GetInCount;
  if Ch_Cnt=0 then exit;    { wenn nix vorhanden, Ende }
  if Ch_Cnt > buffermax then Ch_Cnt:= buffermax;

  ReadFile(hPort, RxBuff, Ch_Cnt, Ch_Read, nil);
  inPtr:= Ch_read;
  outPtr:= 0;
  if inPtr > 0
  then begin
         Result:= RxBuff[0];
         outPtr:= 1;
       end;
end;


function TComPort.ReadB: byte;
begin
  result:= ord(Read);
end;


                  { Flush the port by reading any characters in the queue }
procedure TComPort.Flush;
begin
  inPtr:= 0;
  outPtr:= 0;
  if hPort <> INVALID_HANDLE_VALUE
  then PurgeComm(hPort,
                 PURGE_TXABORT or
                 PURGE_RXABORT or
                 PURGE_TXCLEAR or
                 PURGE_RXCLEAR);
end;

procedure   TComPort.SetLocal(ifLocal: boolean);
begin
  hLocal:= ifLocal;
end;

procedure   TComPort.WaitForEvent (var Event: cardinal);
begin
  WaitCommEvent(hPort, Event, nil);
end;


end.

